#!/bin/bash

# qcg root directory
QCG_BROKER_PATH=@@@SHARE_DIR@@@

# main broker log file
BROKER_LOG_FILE=@@@LOG_DIR@@@/broker-stdouterr

# broker log files to preserve in case of memory error
BROKER_BACKUP_LOGS="@@@LOG_DIR@@@/broker-stdouterr \
					@@@LOG_DIR@@@/qcg-broker.log \
					@@@LOG_DIR@@@/error.log \
					@@@LOG_DIR@@@/qcg-broker-stats.log \
					@@@LOG_DIR@@@/scheduling.log \
					@@@LOG_DIR@@@/transfer.log"

# main service log file
SERVICE_LOG_FILE="@@@LOG_DIR@@@/catalina.out"

# service log files to preserve in case of memory error
SERVICE_BACKUP_LOGS="@@@LOG_DIR@@@/catalina.out \
				  	 @@@LOG_DIR@@@/qcg-broker-service.log"

# debug file
GLOBAL_LOG_FILE="@@@LOG_DIR@@@/memory_monitor.log"

# memory error mail recipients
MAIL_RECIPIENTS=@@@ADMIN_EMAIL@@@


BACKUP_DIR=
LOG_FILE=


detect_memory_error() {
	local log_file=$1
	local component=$2
	local status;

	if [ ! -f "$log_file" ]; then
		echo "Warning: $component log file '$log_file' not exists" >> $GLOBAL_LOG_FILE
		exit 0
	fi

	GREP_RESULT=`grep -i java.lang.OutOfMemoryError $log_file`
	if [ -n "$GREP_RESULT" ]; then
		echo "Detected $component's OutOfMemoryError at" `date "+%F %R"` >> $GLOBAL_LOG_FILE
		status=1
	else
		echo "$component memory OK at " `date "+%F_%R"` >> $GLOBAL_LOG_FILE
		status=0
	fi

	return $status
}


create_backup_dir() {
	local backup_dir
	local date=`date "+%F_%R"`

	backup_dir=@@@VAR_DIR@@@/backup/broker_logs_$date
	mkdir -p $backup_dir

	echo $backup_dir
}


preserve_logs() {
	local log_dir=$1
	local logs="$2"
	local debug_file=$3

	mkdir $log_dir >> $debug_file 2>&1
	for log in $logs; do
		echo "preserving $log ..." >> $debug_file
		cp -R $log $log_dir >> $debug_file 2>&1 
	done
}


find_java_pid() {
	local class_name=$1
	local debug_file=$2
	local pid

	pid=`jps | grep $class_name | cut -d' ' -f 1`
	if [ -z "$pid" ]; then
		return 1
	fi

	echo "found pid [$pid] for class [$class_name] ..." >> $debug_file

	echo $pid

	return 0;
}


dump_heap() {
	local dump_file=$1
	local pid=$2
	local debug_file=$3
	echo "dumping heap memory map to [$dump_file] ..."
	jmap -F -dump:format=b,file=$dump_file $pid >> $debug_file 2>&1
}


send_email() {
	local to="$1"
	local subject="$2"
	local body="$3"
	local debug_file=$4

	echo "$body" | mail -s "$subject" "$to"
}


####
# BROKER
####
detect_memory_error "$BROKER_LOG_FILE" "Broker"
if [ $? -ne 0 ]; then
	BACKUP_DIR=$(create_backup_dir)
	LOG_FILE=$BACKUP_DIR/debug.txt

	# preserve logs
	preserve_logs $BACKUP_DIR/broker-logs "$BROKER_BACKUP_LOGS" $LOG_FILE
	echo "logs preserved ..." >> $LOG_FILE

	# mail info
	hostname=`hostname`
	date=`date "+%F %R"`
	send_email "$MAIL_RECIPIENTS"\
		"QCG broker out ouf memory error" \
		"$date: detected out of memory in QCG Broker installed at $hostname" \
		$LOG_FILE

	# made heap dump
	BROKER_PID=$(find_java_pid FactoryServer $LOG_FILE)
	if [ $? -ne 0 ]; then
		echo "Warning: failed to find broker's PID" >> $LOG_FILE
		echo "all java processes:" >> $LOG_FILE
		jps >> $LOG_FILE
	else
		dump_heap "$BACKUP_DIR/broker_heap_dump.hprof" $BROKER_PID $LOG_FILE
	fi

	# restart broker
	if [ ! -d "$QCG_BROKER_PATH/broker" -o ! -d "$QCG_BROKER_PATH/broker/bin" ]; then
		echo "Warning: failed to restart broker - wrong broker dir [$QCG_BROKER_PATH]" >> $LOG_FILE
	else
		cd $QCG_BROKER_PATH/broker
		./bin/shutdown >> $LOG_FILE
		./bin/startup >> $LOG_FILE
		echo "broker restarted ..." >> $LOG_FILE
	fi
fi


####
# SERVICE
####
detect_memory_error "$SERVICE_LOG_FILE" "Service"
if [ $? -ne 0 ]; then
	if [ -z "$BACKUP_DIR" ]; then
		BACKUP_DIR=$(create_backup_dir)
		LOG_FILE=$BACKUP_DIR/debug.txt
	fi

	# preserve logs
	preserve_logs $BACKUP_DIR/service-logs "$SERVICE_BACKUP_LOGS" $LOG_FILE
	echo "logs preserved ..." >> $LOG_FILE

	# remove old catalina.out
	echo "removing old catalina.out ..." >> $LOG_FILE
	echo ''  2>>$LOG_FILE > "@@@LOG_DIR@@@/catalina.out"

	# mail info
	hostname=`hostname`
	date=`date "+%F %R"`
	send_email "$MAIL_RECIPIENTS" \
		"QCG service out ouf memory error" \
		"$date: detected out of memory in QCG Service installed at $hostname" \
		$LOG_FILE


	# made heap dump
	SERVICE_PID=$(find_java_pid Bootstrap $LOG_FILE)
	if [ $? -ne 0 ]; then
		echo "Warning: failed to find service's PID" >> $LOG_FILE
		echo "all java processes:" >> $LOG_FILE
		jps >> $LOG_FILE
	else
		dump_heap "$BACKUP_DIR/service_heap_dump.hprof" $SERVICE_PID $LOG_FILE
	fi

	# restart service
	if [ ! -d "$QCG_BROKER_PATH/service" -o ! -d "$QCG_BROKER_PATH/service/bin" ]; then
		echo "Warning: failed to restart service - wrong QCG dir [$QCG_BROKER_PATH]" >> $LOG_FILE
	else
		cd $QCG_BROKER_PATH/service
		./bin/shutdown.sh >> $LOG_FILE
		./bin/startup.sh >> $LOG_FILE
		echo "service restarted ..." >> $LOG_FILE
	fi
fi
